; "Artefacts"/"Artifacts" by Plush

; Code by Gunnar Ruthenberg - Krill (Krill@Plush.de),
; Music by Alexander Rotzsch - Fanta (Fanta@Plush.de)

; released at the C64 4K Demo Competition at Breakpoint 2006


.include "cpu.inc"
.include "vic.inc"
.include "sid.inc"
.include "cia.inc"

.include "screen.s"
.include "codegen.s"
.include "texgen.s"
.include "spheres.s"
.include "voxels.s"
.include "yrotator.s"
.include "interlude.s"
.include "irqhandler.s"
.include "music.s"
.include "math.s"

.include "polyx.s"; generated by polyx.m

.include "strings.inc"
.include "zp.inc"

.define CLRSCR $e544

.CODE
                .word *+$02; loading address
                
                ; entry point
                ; interrupts are disabled here

Artefacts:      
.ifdef NTSC_VERSION
                .res $10,OPC_NOP; here to prevent the surface scene from crashing (code modifier conflict)
.endif
                jsr CLRSCR

                SUB_INITMATH; math.s

                lda #MEMCONFIG_IO; 64K RAM minus IO at $d000-$dfff
                sta IO_PORT

                SUB_INITMUSIC; music.s

                SUB_SETUPSCREENMODE; screen.s

                SUB_GENCSET; screen.s

                lda CIA1_ICR
				inc VIC2_IRR
                cli
                
sceneloop:      lda interludespeed
                jsr startinterlude
                PLANETSCENE; spheres.s
                SURFACESCENE; voxels.s
                ARTEFACTSCENE; yrotator.s
                lda #altcolours-coloursets
                eor COLOURSCHEME
                sta COLOURSCHEME
                lda #stringloop-strings-$01
                sta INTERLUDECOUNT
                jmp sceneloop

                ; subroutines
                SUB_INTERLUDE; interlude.s
                SUB_CODEGENERATOR; codegen.s
                SUB_SCREEN; screen.s
                SUB_TEXTUREGENERATOR; texgen.s
                SUB_DRAWTEXTURESPHERE; spheres.s
                SUB_MATH; math.s

                ; irq handler
                IRQ_HANDLER; irqhandler.s

                ; templates for generated code
codefragments = *-$0a; anti-padding
                FRG_SPHERECODE; sphere.s
                FRG_VOXELCODE; voxels.s
                FRG_YROTCODE; yrotator.s

                ; data
.DATA
                .ASSERT (>tablespolyend) = (>tablespolynom), error, "tables polynomials page boundary crossing"

tablespolynom:  DAT_POLYNOMIALS; polyx.s
tablespolyend:                
                DAT_TABLES; math.s
                DAT_COLOURS; screen.s
                DAT_DITHERING; screen.s

                DAT_MUSIC; music.s
                
                TXT_STRINGS; strings.inc

                ; generated tables
                DAT_SINCOSTABLERAW; math.s
                DAT_SPHEREMAPPINGTABLE; spheres.s
                DAT_ARCTANTABLE; math.s
                DAT_YROTXYTABLES; yrotator.s

.segment "TABLES"
tables:
.END